@@ -410,12 +410,10 @@ if redis.call("hexists", listings_key, job_key) == 1 then
             -- Owner is the same, leave it alone...
             redis.call("set", last_modified_key, last_modified_blob)
             apply_ttl(owner_key, ms_expiry)
-            result["status"] = "${ok}"
-        else
-            result["status"] = "${error}"
-            result["reason"] = "${already_claimed}"
-            result["owner"] = owner
         end
+        result["status"] = "${error}"
+        result["reason"] = "${already_claimed}"
+        result["owner"] = owner
     else
         redis.call("set", owner_key, expected_owner)
         redis.call("set", last_modified_key, last_modified_blob)
@@ -817,9 +815,9 @@ return cmsgpack.pack(result)
                 job_details = job_data.get('details', {})
             except (ValueError, TypeError, KeyError):
                 with excutils.save_and_reraise_exception():
-                    LOG.warn("Incorrectly formatted job data found at"
-                             " key: %s[%s]", self.listings_key,
-                             raw_job_key, exc_info=True)
+                    LOG.warning("Incorrectly formatted job data found at"
+                                " key: %s[%s]", self.listings_key,
+                                raw_job_key, exc_info=True)
             else:
                 postings.append(RedisJob(self, job_name, job_sequence_id,
                                          raw_job_key, uuid=job_uuid,
