@@ -14,16 +14,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from concurrent import futures
 from oslo_utils import uuidutils
 
 from taskflow.engines.action_engine import executor
 from taskflow.engines.worker_based import protocol as pr
 from taskflow import exceptions as excp
 from taskflow import test
+from taskflow.test import mock
 from taskflow.tests import utils
 from taskflow.types import failure
-from taskflow.types import timing
 
 
 class TestProtocolValidation(test.TestCase):
@@ -51,9 +50,10 @@ class TestProtocolValidation(test.TestCase):
                           pr.Notify.validate, msg, True)
 
     def test_request(self):
-        msg = pr.Request(utils.DummyTask("hi"), uuidutils.generate_uuid(),
-                         pr.EXECUTE, {}, 1.0)
-        pr.Request.validate(msg.to_dict())
+        request = pr.Request(utils.DummyTask("hi"),
+                             uuidutils.generate_uuid(),
+                             pr.EXECUTE, {}, 1.0)
+        pr.Request.validate(request.to_dict())
 
     def test_request_invalid(self):
         msg = {
@@ -64,11 +64,12 @@ class TestProtocolValidation(test.TestCase):
         self.assertRaises(excp.InvalidFormat, pr.Request.validate, msg)
 
     def test_request_invalid_action(self):
-        msg = pr.Request(utils.DummyTask("hi"), uuidutils.generate_uuid(),
-                         pr.EXECUTE, {}, 1.0)
-        msg = msg.to_dict()
-        msg['action'] = 'NOTHING'
-        self.assertRaises(excp.InvalidFormat, pr.Request.validate, msg)
+        request = pr.Request(utils.DummyTask("hi"),
+                             uuidutils.generate_uuid(),
+                             pr.EXECUTE, {}, 1.0)
+        request = request.to_dict()
+        request['action'] = 'NOTHING'
+        self.assertRaises(excp.InvalidFormat, pr.Request.validate, request)
 
     def test_response_progress(self):
         msg = pr.Response(pr.EVENT, details={'progress': 0.5},
@@ -94,8 +95,6 @@ class TestProtocol(test.TestCase):
 
     def setUp(self):
         super(TestProtocol, self).setUp()
-        timing.StopWatch.set_now_override()
-        self.addCleanup(timing.StopWatch.clear_overrides)
         self.task = utils.DummyTask()
         self.task_uuid = 'task-uuid'
         self.task_action = 'execute'
@@ -107,7 +106,6 @@ class TestProtocol(test.TestCase):
                               uuid=self.task_uuid,
                               action=self.task_action,
                               arguments=self.task_args,
-                              progress_callback=None,
                               timeout=self.timeout)
         request_kwargs.update(kwargs)
         return pr.Request(**request_kwargs)
@@ -123,8 +121,8 @@ class TestProtocol(test.TestCase):
 
     def test_request_transitions(self):
         request = self.request()
-        self.assertEqual(pr.WAITING, request.state)
-        self.assertIn(request.state, pr.WAITING_STATES)
+        self.assertEqual(pr.WAITING, request.current_state)
+        self.assertIn(request.current_state, pr.WAITING_STATES)
         self.assertRaises(excp.InvalidState, request.transition, pr.SUCCESS)
         self.assertFalse(request.transition(pr.WAITING))
         self.assertTrue(request.transition(pr.PENDING))
@@ -135,54 +133,71 @@ class TestProtocol(test.TestCase):
 
     def test_creation(self):
         request = self.request()
-        self.assertEqual(request.uuid, self.task_uuid)
-        self.assertEqual(request.task, self.task)
-        self.assertIsInstance(request.result, futures.Future)
-        self.assertFalse(request.result.done())
+        self.assertEqual(self.task_uuid, request.uuid)
+        self.assertEqual(self.task, request.task)
+        self.assertFalse(request.future.done())
 
     def test_to_dict_default(self):
-        self.assertEqual(self.request().to_dict(), self.request_to_dict())
+        request = self.request()
+        self.assertEqual(self.request_to_dict(), request.to_dict())
 
     def test_to_dict_with_result(self):
-        self.assertEqual(self.request(result=333).to_dict(),
-                         self.request_to_dict(result=('success', 333)))
+        request = self.request(result=333)
+        self.assertEqual(self.request_to_dict(result=('success', 333)),
+                         request.to_dict())
 
     def test_to_dict_with_result_none(self):
-        self.assertEqual(self.request(result=None).to_dict(),
-                         self.request_to_dict(result=('success', None)))
+        request = self.request(result=None)
+        self.assertEqual(self.request_to_dict(result=('success', None)),
+                         request.to_dict())
 
     def test_to_dict_with_result_failure(self):
         a_failure = failure.Failure.from_exception(RuntimeError('Woot!'))
         expected = self.request_to_dict(result=('failure',
                                                 a_failure.to_dict()))
-        self.assertEqual(self.request(result=a_failure).to_dict(), expected)
+        request = self.request(result=a_failure)
+        self.assertEqual(expected, request.to_dict())
 
     def test_to_dict_with_failures(self):
         a_failure = failure.Failure.from_exception(RuntimeError('Woot!'))
         request = self.request(failures={self.task.name: a_failure})
         expected = self.request_to_dict(
             failures={self.task.name: a_failure.to_dict()})
-        self.assertEqual(request.to_dict(), expected)
+        self.assertEqual(expected, request.to_dict())
 
-    def test_pending_not_expired(self):
-        req = self.request()
-        timing.StopWatch.set_offset_override(self.timeout - 1)
-        self.assertFalse(req.expired)
+    def test_to_dict_with_invalid_json_failures(self):
+        exc = RuntimeError(Exception("I am not valid JSON"))
+        a_failure = failure.Failure.from_exception(exc)
+        request = self.request(failures={self.task.name: a_failure})
+        expected = self.request_to_dict(
+            failures={self.task.name: a_failure.to_dict(include_args=False)})
+        self.assertEqual(expected, request.to_dict())
 
-    def test_pending_expired(self):
-        req = self.request()
-        timing.StopWatch.set_offset_override(self.timeout + 1)
-        self.assertTrue(req.expired)
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_pending_not_expired(self, now):
+        now.return_value = 0
+        request = self.request()
+        now.return_value = self.timeout - 1
+        self.assertFalse(request.expired)
 
-    def test_running_not_expired(self):
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_pending_expired(self, now):
+        now.return_value = 0
+        request = self.request()
+        now.return_value = self.timeout + 1
+        self.assertTrue(request.expired)
+
+    @mock.patch('oslo_utils.timeutils.now')
+    def test_running_not_expired(self, now):
+        now.return_value = 0
         request = self.request()
         request.transition(pr.PENDING)
         request.transition(pr.RUNNING)
-        timing.StopWatch.set_offset_override(self.timeout + 1)
+        now.return_value = self.timeout + 1
         self.assertFalse(request.expired)
 
     def test_set_result(self):
         request = self.request()
         request.set_result(111)
-        result = request.result.result()
-        self.assertEqual(result, (executor.EXECUTED, 111))
+        result = request.future.result()
+        self.assertEqual((executor.EXECUTED, 111), result)
