@@ -14,15 +14,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import copy
 import os
 import sys
 import traceback
 
+from oslo_utils import encodeutils
 from oslo_utils import reflection
 import six
 
 from taskflow import exceptions as exc
+from taskflow.utils import iter_utils
+from taskflow.utils import mixins
+from taskflow.utils import schema_utils as su
+
+
+_exception_message = encodeutils.exception_to_unicode
 
 
 def _copy_exc_info(exc_info):
@@ -35,23 +43,6 @@ def _copy_exc_info(exc_info):
     return (exc_type, copy.copy(exc_value), tb)
 
 
-def _fill_iter(it, desired_len, filler=None):
-    """Iterates over a provided iterator up to the desired length.
-
-    If the source iterator does not have enough values then the filler
-    value is yielded until the desired length is reached.
-    """
-    count = 0
-    for value in it:
-        if count >= desired_len:
-            return
-        yield value
-        count += 1
-    while count < desired_len:
-        yield filler
-        count += 1
-
-
 def _are_equal_exc_info_tuples(ei1, ei2):
     if ei1 == ei2:
         return True
@@ -64,7 +55,7 @@ def _are_equal_exc_info_tuples(ei1, ei2):
     if ei1[0] is not ei2[0]:
         return False
     if not all((type(ei1[1]) == type(ei2[1]),
-                exc.exception_message(ei1[1]) == exc.exception_message(ei2[1]),
+                _exception_message(ei1[1]) == _exception_message(ei2[1]),
                 repr(ei1[1]) == repr(ei2[1]))):
         return False
     if ei1[2] == ei2[2]:
@@ -74,14 +65,14 @@ def _are_equal_exc_info_tuples(ei1, ei2):
     return tb1 == tb2
 
 
-class Failure(object):
+class Failure(mixins.StrMixin):
     """An immutable object that represents failure.
 
     Failure objects encapsulate exception information so that they can be
     re-used later to re-raise, inspect, examine, log, print, serialize,
     deserialize...
 
-    One example where they are dependened upon is in the WBE engine. When a
+    One example where they are depended upon is in the WBE engine. When a
     remote worker throws an exception, the WBE based engine will receive that
     exception and desire to reraise it to the user/caller of the WBE based
     engine for appropriate handling (this matches the behavior of non-remote
@@ -126,12 +117,64 @@ class Failure(object):
       to have code ran when this happens, and this can cause issues and
       side-effects that the receiver would not have intended to have caused).
 
-    TODO(harlowja): when/if http://bugs.python.org/issue17911 merges and
-    becomes available for use we should be able to use that and simplify the
-    methods and contents of this object.
+    TODO(harlowja): use parts of http://bugs.python.org/issue17911 and the
+    backport at https://pypi.python.org/pypi/traceback2/ to (hopefully)
+    simplify the methods and contents of this object...
     """
     DICT_VERSION = 1
 
+    BASE_EXCEPTIONS = ('BaseException', 'Exception')
+    """
+    Root exceptions of all other python exceptions.
+
+    See: https://docs.python.org/2/library/exceptions.html
+    """
+
+    #: Expected failure schema (in json schema format).
+    SCHEMA = {
+        "$ref": "#/definitions/cause",
+        "definitions": {
+            "cause": {
+                "type": "object",
+                'properties': {
+                    'version': {
+                        "type": "integer",
+                        "minimum": 0,
+                    },
+                    'exc_args': {
+                        "type": "array",
+                        "minItems": 0,
+                    },
+                    'exception_str': {
+                        "type": "string",
+                    },
+                    'traceback_str': {
+                        "type": "string",
+                    },
+                    'exc_type_names': {
+                        "type": "array",
+                        "items": {
+                            "type": "string",
+                        },
+                        "minItems": 1,
+                    },
+                    'causes': {
+                        "type": "array",
+                        "items": {
+                            "$ref": "#/definitions/cause",
+                        },
+                    }
+                },
+                "required": [
+                    "exception_str",
+                    'traceback_str',
+                    'exc_type_names',
+                ],
+                "additionalProperties": True,
+            },
+        },
+    }
+
     def __init__(self, exc_info=None, **kwargs):
         if not kwargs:
             if exc_info is None:
@@ -144,16 +187,20 @@ class Failure(object):
                     raise ValueError("Provided 'exc_info' must contain three"
                                      " elements")
             self._exc_info = exc_info
+            self._exc_args = tuple(getattr(exc_info[1], 'args', []))
             self._exc_type_names = tuple(
                 reflection.get_all_class_names(exc_info[0], up_to=Exception))
             if not self._exc_type_names:
                 raise TypeError("Invalid exception type '%s' (%s)"
                                 % (exc_info[0], type(exc_info[0])))
-            self._exception_str = exc.exception_message(self._exc_info[1])
+            self._exception_str = _exception_message(self._exc_info[1])
             self._traceback_str = ''.join(
                 traceback.format_tb(self._exc_info[2]))
+            self._causes = kwargs.pop('causes', None)
         else:
-            self._exc_info = exc_info  # may be None
+            self._causes = kwargs.pop('causes', None)
+            self._exc_info = exc_info
+            self._exc_args = tuple(kwargs.pop('exc_args', []))
             self._exception_str = kwargs.pop('exception_str')
             self._exc_type_names = tuple(kwargs.pop('exc_type_names', []))
             self._traceback_str = kwargs.pop('traceback_str', None)
@@ -165,14 +212,47 @@ class Failure(object):
     @classmethod
     def from_exception(cls, exception):
         """Creates a failure object from a exception instance."""
-        return cls((type(exception), exception, None))
+        exc_info = (
+            type(exception),
+            exception,
+            getattr(exception, '__traceback__', None)
+        )
+        return cls(exc_info=exc_info)
+
+    @classmethod
+    def validate(cls, data):
+        """Validate input data matches expected failure ``dict`` format."""
+        try:
+            su.schema_validate(data, cls.SCHEMA)
+        except su.ValidationError as e:
+            raise exc.InvalidFormat("Failure data not of the"
+                                    " expected format: %s" % (e.message), e)
+        else:
+            # Ensure that all 'exc_type_names' originate from one of
+            # BASE_EXCEPTIONS, because those are the root exceptions that
+            # python mandates/provides and anything else is invalid...
+            causes = collections.deque([data])
+            while causes:
+                cause = causes.popleft()
+                root_exc_type = cause['exc_type_names'][-1]
+                if root_exc_type not in cls.BASE_EXCEPTIONS:
+                    raise exc.InvalidFormat(
+                        "Failure data 'exc_type_names' must"
+                        " have an initial exception type that is one"
+                        " of %s types: '%s' is not one of those"
+                        " types" % (cls.BASE_EXCEPTIONS, root_exc_type))
+                sub_causes = cause.get('causes')
+                if sub_causes:
+                    causes.extend(sub_causes)
 
     def _matches(self, other):
         if self is other:
             return True
         return (self._exc_type_names == other._exc_type_names
+                and self.exception_args == other.exception_args
                 and self.exception_str == other.exception_str
-                and self.traceback_str == other.traceback_str)
+                and self.traceback_str == other.traceback_str
+                and self.causes == other.causes)
 
     def matches(self, other):
         """Checks if another object is equivalent to this object.
@@ -205,7 +285,7 @@ class Failure(object):
 
     @property
     def exception(self):
-        """Exception value, or None if exception value is not present.
+        """Exception value, or none if exception value is not present.
 
         Exception value may be lost during serialization.
         """
@@ -219,9 +299,19 @@ class Failure(object):
         """String representation of exception."""
         return self._exception_str
 
+    @property
+    def exception_args(self):
+        """Tuple of arguments given to the exception constructor."""
+        return self._exc_args
+
     @property
     def exc_info(self):
-        """Exception info tuple or None."""
+        """Exception info tuple or none.
+
+        See: https://docs.python.org/2/library/sys.html#sys.exc_info for what
+             the contents of this tuple are (if none, then no contents can
+             be examined).
+        """
         return self._exc_info
 
     @property
@@ -233,13 +323,15 @@ class Failure(object):
     def reraise_if_any(failures):
         """Re-raise exceptions if argument is not empty.
 
-        If argument is empty list, this method returns None. If
-        argument is a list with a single ``Failure`` object in it,
-        that failure is reraised. Else, a
+        If argument is empty list/tuple/iterator, this method returns
+        None. If argument is converted into a list with a
+        single ``Failure`` object in it, that failure is reraised. Else, a
         :class:`~taskflow.exceptions.WrappedFailure` exception
-        is raised with a failure list as causes.
+        is raised with the failure list as causes.
         """
-        failures = list(failures)
+        if not isinstance(failures, (list, tuple)):
+            # Convert generators/other into a list...
+            failures = list(failures)
         if len(failures) == 1:
             failures[0].reraise()
         elif len(failures) > 1:
@@ -269,7 +361,67 @@ class Failure(object):
                 return cls
         return None
 
-    def __str__(self):
+    @classmethod
+    def _extract_causes_iter(cls, exc_val):
+        seen = [exc_val]
+        causes = [exc_val]
+        while causes:
+            exc_val = causes.pop()
+            if exc_val is None:
+                continue
+            # See: https://www.python.org/dev/peps/pep-3134/ for why/what
+            # these are...
+            #
+            # '__cause__' attribute for explicitly chained exceptions
+            # '__context__' attribute for implicitly chained exceptions
+            # '__traceback__' attribute for the traceback
+            #
+            # See: https://www.python.org/dev/peps/pep-0415/ for why/what
+            # the '__suppress_context__' is/means/implies...
+            suppress_context = getattr(exc_val,
+                                       '__suppress_context__', False)
+            if suppress_context:
+                attr_lookups = ['__cause__']
+            else:
+                attr_lookups = ['__cause__', '__context__']
+            nested_exc_val = None
+            for attr_name in attr_lookups:
+                attr_val = getattr(exc_val, attr_name, None)
+                if attr_val is None:
+                    continue
+                if attr_val not in seen:
+                    nested_exc_val = attr_val
+                    break
+            if nested_exc_val is not None:
+                exc_info = (
+                    type(nested_exc_val),
+                    nested_exc_val,
+                    getattr(nested_exc_val, '__traceback__', None),
+                )
+                seen.append(nested_exc_val)
+                causes.append(nested_exc_val)
+                yield cls(exc_info=exc_info)
+
+    @property
+    def causes(self):
+        """Tuple of all *inner* failure *causes* of this failure.
+
+        NOTE(harlowja): Does **not** include the current failure (only
+        returns connected causes of this failure, if any). This property
+        is really only useful on 3.x or newer versions of python as older
+        versions do **not** have associated causes (the tuple will **always**
+        be empty on 2.x versions of python).
+
+        Refer to :pep:`3134` and :pep:`409` and :pep:`415` for what
+        this is examining to find failure causes.
+        """
+        if self._causes is not None:
+            return self._causes
+        else:
+            self._causes = tuple(self._extract_causes_iter(self.exception))
+            return self._causes
+
+    def __unicode__(self):
         return self.pformat()
 
     def pformat(self, traceback=False):
@@ -309,6 +461,12 @@ class Failure(object):
 
     def __setstate__(self, dct):
         self._exception_str = dct['exception_str']
+        if 'exc_args' in dct:
+            self._exc_args = tuple(dct['exc_args'])
+        else:
+            # Guess we got an older version somehow, before this
+            # was added, so at that point just set to an empty tuple...
+            self._exc_args = ()
         self._traceback_str = dct['traceback_str']
         self._exc_type_names = dct['exc_type_names']
         if 'exc_info' in dct:
@@ -320,9 +478,13 @@ class Failure(object):
             # what the twisted people have done, see for example
             # twisted-13.0.0/twisted/python/failure.py#L89 for how they
             # created a fake traceback object...
-            self._exc_info = tuple(_fill_iter(dct['exc_info'], 3))
+            self._exc_info = tuple(iter_utils.fill(dct['exc_info'], 3))
         else:
             self._exc_info = None
+        causes = dct.get('causes')
+        if causes is not None:
+            causes = tuple(self.from_dict(d) for d in causes)
+        self._causes = causes
 
     @classmethod
     def from_dict(cls, data):
@@ -332,15 +494,24 @@ class Failure(object):
         if version != cls.DICT_VERSION:
             raise ValueError('Invalid dict version of failure object: %r'
                              % version)
+        causes = data.get('causes')
+        if causes is not None:
+            data['causes'] = tuple(cls.from_dict(d) for d in causes)
         return cls(**data)
 
-    def to_dict(self):
-        """Converts this object to a dictionary."""
+    def to_dict(self, include_args=True):
+        """Converts this object to a dictionary.
+
+        :param include_args: boolean indicating whether to include the
+                             exception args in the output.
+        """
         return {
             'exception_str': self.exception_str,
             'traceback_str': self.traceback_str,
             'exc_type_names': list(self),
             'version': self.DICT_VERSION,
+            'exc_args': self.exception_args if include_args else tuple(),
+            'causes': [f.to_dict() for f in self.causes],
         }
 
     def copy(self):
@@ -348,4 +519,6 @@ class Failure(object):
         return Failure(exc_info=_copy_exc_info(self.exc_info),
                        exception_str=self.exception_str,
                        traceback_str=self.traceback_str,
-                       exc_type_names=self._exc_type_names[:])
+                       exc_args=self.exception_args,
+                       exc_type_names=self._exc_type_names[:],
+                       causes=self._causes)
