@@ -37,27 +37,7 @@ def _parse_hosts(hosts):
 
 
 def prettify_failures(failures, limit=-1):
-    """Prettifies a checked commits failures (ignores sensitive data...).
-
-    Example input and output:
-
-    >>> from taskflow.utils import kazoo_utils
-    >>> conf = {"hosts": ['localhost:2181']}
-    >>> c = kazoo_utils.make_client(conf)
-    >>> c.start(timeout=1)
-    >>> txn = c.transaction()
-    >>> txn.create("/test")
-    >>> txn.check("/test", 2)
-    >>> txn.delete("/test")
-    >>> try:
-    ...     kazoo_utils.checked_commit(txn)
-    ... except kazoo_utils.KazooTransactionException as e:
-    ...     print(kazoo_utils.prettify_failures(e.failures, limit=1))
-    ...
-    RolledBackError@Create(path='/test') and 2 more...
-    >>> c.stop()
-    >>> c.close()
-    """
+    """Prettifies a checked commits failures (ignores sensitive data...)."""
     prettier = []
     for (op, r) in failures:
         pretty_op = reflection.get_class_name(op, fully_qualified=False)
@@ -129,13 +109,7 @@ def checked_commit(txn):
 def finalize_client(client):
     """Stops and closes a client, even if it wasn't started."""
     client.stop()
-    try:
-        client.close()
-    except TypeError:
-        # NOTE(harlowja): https://github.com/python-zk/kazoo/issues/167
-        #
-        # This can be removed after that one is fixed/merged.
-        pass
+    client.close()
 
 
 def check_compatible(client, min_version=None, max_version=None):
@@ -171,7 +145,43 @@ def check_compatible(client, min_version=None, max_version=None):
 
 
 def make_client(conf):
-    """Creates a kazoo client given a configuration dictionary."""
+    """Creates a `kazoo`_ `client`_ given a configuration dictionary.
+
+    :param conf: configuration dictionary that will be used to configure
+                 the created client
+    :type conf: dict
+
+    The keys that will be extracted are:
+
+    - ``read_only``: boolean that specifies whether to allow connections to
+      read only servers, defaults to ``False``
+    - ``randomize_hosts``: boolean that specifies whether to randomize
+      host lists provided, defaults to ``False``
+    - ``command_retry``: a kazoo `retry`_ object (or dict of options which
+      will be used for creating one) that will be used for retrying commands
+      that are executed
+    - ``connection_retry``: a kazoo `retry`_ object (or dict of options which
+      will be used for creating one)  that will be used for retrying
+      connection failures that occur
+    - ``hosts``: a string, list, set (or dict with host keys) that will
+      specify the hosts the kazoo client should be connected to, if none
+      is provided then ``localhost:2181`` will be used by default
+    - ``timeout``: a float value that specifies the default timeout that the
+      kazoo client will use
+    - ``handler``: a kazoo handler object that can be used to provide the
+      client with alternate async strategies (the default is `thread`_
+      based, but `gevent`_, or `eventlet`_ ones can be provided as needed)
+
+    .. _client: http://kazoo.readthedocs.org/en/latest/api/client.html
+    .. _kazoo: http://kazoo.readthedocs.org/
+    .. _retry: http://kazoo.readthedocs.org/en/latest/api/retry.html
+    .. _gevent: http://kazoo.readthedocs.org/en/latest/api/\
+                handlers/gevent.html
+    .. _eventlet: http://kazoo.readthedocs.org/en/latest/api/\
+                  handlers/eventlet.html
+    .. _thread: http://kazoo.readthedocs.org/en/latest/api/\
+                handlers/threading.html
+    """
     # See: http://kazoo.readthedocs.org/en/latest/api/client.html
     client_kwargs = {
         'read_only': bool(conf.get('read_only')),
